import 'package:car_app/exception/key_not_found.dart';
import 'package:car_app/exception/server.dart';
import 'package:car_app/model/route.dart';
import 'package:car_app/service/interface/key_value_storage.dart';
import 'package:car_app/service/interface/route.dart';

class CachedRouteService extends RouteService {
  final RouteService routeService;
  final KeyValueStorageService<String, List<Route>> storageService;
  final String storageKey = "routes";

  CachedRouteService({required this.routeService, required this.storageService});

  @override
  Future<Route> add(Route route) async {
    var isSynchronized = await _synchronize();

    if(isSynchronized) {
      try {
        return routeService.add(route);
      } on ServerException catch (_) {
        await _addRouteToStorage(route);
        return route;
      }
    }
    await _addRouteToStorage(route);
    return route;
  }

  Future<bool> _synchronize() async {
    List<Route> unsuccessfulRoutes = [];
    try {
      var routes = await storageService.get(storageKey);
      
      for (var route in routes) {
        try {
          await routeService.add(route);
          return true;
        } on ServerException catch (_) {
          unsuccessfulRoutes.add(route);
        }
      }

    } on KeyNotFoundException catch (_) {} // No routes stored yet

    storageService.add(storageKey, unsuccessfulRoutes);
    return false;
  }

  Future<void> _addRouteToStorage(Route route) async {
    List<Route> routes = [];
      try {
        routes = await storageService.get(storageKey);
      } on KeyNotFoundException catch (_) {} // Ignore exception, as no routes are stored yet

      routes.add(route);
      return storageService.add(storageKey, routes);
  }
}
